<?php
/**
 * Handle custom permalink structure for events
 */

// Exit if accessed directly
if (!defined('ABSPATH')) exit;

class Vizou_Events_Permalink_Handler {
    private static $instance = null;
    private static $is_filtering = false;

    public static function register() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        add_action('init', [$this, 'add_rewrite_rules']);
        add_filter('post_type_link', [$this, 'filter_event_permalink'], 10, 2);
        add_action('parse_request', [$this, 'parse_request']);
        add_filter('template_include', [$this, 'template_include']);
    }

    public function add_rewrite_rules() {
        add_rewrite_rule(
            'event/(.+?)-(.+?)-([0-9]{4})-([0-9]{2})-([0-9]{2})/?$',
            'index.php?post_type=event&name=$matches[1]&artist=$matches[2]&year=$matches[3]&monthnum=$matches[4]&day=$matches[5]',
            'top'
        );
        
        // Keep old rule for backward compatibility
        add_rewrite_rule(
            'event/(.+?)-([0-9]{4})-([0-9]{2})-([0-9]{2})/?$',
            'index.php?post_type=event&name=$matches[1]&year=$matches[2]&monthnum=$matches[3]&day=$matches[4]',
            'top'
        );
    }

    public function filter_event_permalink($post_link, $post) {
        if (self::$is_filtering || !is_object($post) || $post->post_type !== 'event') {
            return $post_link;
        }

        self::$is_filtering = true;

        // Get event start date - ACF returns in d/m/Y format
        $start_date = get_field('event_start', $post->ID);
        if (!$start_date) {
            self::$is_filtering = false;
            return $post_link;
        }

        // Get artist name
        $artist_name = '';
        $artist_terms = get_the_terms($post->ID, 'artist');
        if ($artist_terms && !is_wp_error($artist_terms) && !empty($artist_terms)) {
            $artist_name = sanitize_title($artist_terms[0]->name);
        }

        // Convert d/m/Y to Y-m-d for URL
        $date_parts = explode('/', $start_date);
        if (count($date_parts) !== 3) {
            self::$is_filtering = false;
            return $post_link;
        }

        $date_formatted = sprintf('%04d-%02d-%02d', 
            intval($date_parts[2]),  // Year
            intval($date_parts[1]),  // Month
            intval($date_parts[0])   // Day
        );
        
        // Include artist in URL if available
        if (!empty($artist_name)) {
            $permalink = preg_replace(
                '|/event/([^/]+)|',
                '/event/$1-' . $artist_name . '-' . $date_formatted,
                $post_link
            );
        } else {
            // Fallback to original format if no artist
            $permalink = preg_replace(
                '|/event/([^/]+)|',
                '/event/$1-' . $date_formatted,
                $post_link
            );
        }

        self::$is_filtering = false;
        return $permalink;
    }

    public function parse_request($wp) {
        // Ensure this only runs for event URLs
        if (strpos($_SERVER['REQUEST_URI'], '/event/') !== 0) {
            return;
        }

        $request_uri = $_SERVER['REQUEST_URI'];
        
        // Check for new format with artist (venue-artist-date)
        $pattern_with_artist = '/^\/event\/(.+?)-(.+?)-(\d{4})-(\d{2})-(\d{2})\/?$/';
        
        // Check for old format without artist (venue-date)
        $pattern_without_artist = '/^\/event\/(.+?)-(\d{4})-(\d{2})-(\d{2})\/?$/';
        
        $slug = '';
        
        if (preg_match($pattern_with_artist, $request_uri, $matches)) {
            $slug = $matches[1];
        } elseif (preg_match($pattern_without_artist, $request_uri, $matches)) {
            $slug = $matches[1];
        }
        
        if (!empty($slug)) {
            $args = array(
                'name' => $slug,
                'post_type' => 'event',
                'posts_per_page' => 1
            );
            
            $posts = get_posts($args);
            
            if (!empty($posts)) {
                // Directly set the global $post
                global $post;
                $post = $posts[0];
                setup_postdata($post);

                // Modify query variables more aggressively
                $wp->query_vars = array(
                    'post_type' => 'event',
                    'name' => $slug,
                    'p' => $post->ID
                );

                // Optional: Force WordPress to recognize this as a single post
                add_filter('pre_get_posts', function($query) {
                    if (!is_admin() && $query->is_main_query()) {
                        $query->set('post_type', 'event');
                        $query->is_single = true;
                        $query->is_singular = true;
                    }
                    return $query;
                }, 99);
            }
        }
    }

    public function template_include($template) {
        if (is_singular('event')) {
            // First, check for a theme override as single-event.php
            $theme_template = locate_template(['single-event.php']);
            if ($theme_template) {
                return $theme_template;
            }
            
            // Next, check for a theme override in vizou-events folder
            $theme_template = locate_template(['vizou-events/vizou-single-event.php']);
            if ($theme_template) {
                return $theme_template;
            }
            
            // Fallback to plugin template
            $plugin_template = plugin_dir_path(dirname(__FILE__)) . 'templates/vizou-single-event.php';
            if (file_exists($plugin_template)) {
                return $plugin_template;
            }
        }
        
        return $template;
    }
}

// Initialize permalink handler
add_action('init', ['Vizou_Events_Permalink_Handler', 'register']);