<?php
/**
 * Template: Single Event Template
 *
 * This template displays a single event on its dedicated page.
 * To customize, copy this file to your-theme/single-event.php or your-theme/vizou-events/vizou-single-event.php
 * 
 * This template includes the WordPress header and footer and handles the main query.
 *
 * Available template tags:
 * 
 * Event Date/Time:
 * - vizou_event_date($post_id = null, $echo = true) - Displays the event's start date
 * - vizou_event_end_date($post_id = null, $echo = true) - Displays the event's end date (for multi-day events)
 * - vizou_event_time($post_id = null, $echo = true) - Displays the event's time
 * 
 * Artist Information:
 * - vizou_event_artist($post_id = null, $link = true, $echo = true) - Displays the artist with optional link
 * - vizou_artist_url($post_id = null) - Returns the artist URL
 * 
 * Venue Information:
 * - vizou_event_venue($post_id = null, $link = true, $echo = true) - Displays the venue with optional link
 * - vizou_venue_url($post_id = null) - Returns the venue URL
 * - vizou_venue_street($post_id = null, $echo = true) - Displays the venue street address
 * - vizou_venue_city($post_id = null, $echo = true) - Displays the venue city
 * - vizou_venue_province($post_id = null, $echo = true) - Displays the venue province/state
 * - vizou_venue_country($post_id = null, $echo = true) - Displays the venue country
 * 
 * Event Content:
 * - vizou_event_notes($post_id = null, $echo = true) - Displays the event description/notes
 * - vizou_event_status($post_id = null, $echo = true) - Displays the event status (if not "Active")
 * - vizou_event_tickets_link($post_id = null, $echo = true) - Displays the tickets button with link
 * - vizou_ticket_url($post_id = null) - Returns the ticket URL
 * 
 * Helper Functions:
 * - vizou_is_past_event($post_id = null) - Returns true if this is a past event
 * - vizou_is_cancelled_event($post_id = null) - Returns true if this is a cancelled event
 *
 * @package Vizou_Events_Manager
 * @since 0.1.5
 */

// Exit if accessed directly
if (!defined('ABSPATH')) exit;

get_header(); 
?>
	
    <?php if (have_posts()) : while (have_posts()) : the_post(); ?>

        <div class="grid">
            <main class="content">

                <?php
                $thumb = wp_get_attachment_image_src(get_post_thumbnail_id(), 'large', false);
                $thumb_id = absint(get_post_thumbnail_id());
                $thumb_alt = '';
                if ($thumb_id) {
                    $thumb_alt = sanitize_text_field(get_post_meta($thumb_id, '_wp_attachment_image_alt', true));			
                    if (empty($thumb_alt)) {
                        $attachment = get_post($thumb_id);
                        if ($attachment) {
                            $thumb_alt = sanitize_text_field($attachment->post_title);
                        }
                    }
                }
                ?>

                <!-- BEGIN EVENT -->

                <article class="event">

                    <p class="which-template">Vizou single event</p>

                    <!-- TICKETS -->
                    <p class="event-tickets">
                        <?php vizou_event_tickets_link(); ?>
                    </p>

                    <!-- DATE, TIME -->
                   <p class="event-date">
                        <?php 
                        // Get both dates to determine if this is a multi-day event
                        $start_date = vizou_event_date(null, false);
                        $end_date = vizou_event_end_date(null, false);
                        
                        // Output date or date range appropriately
                        echo esc_html($start_date);
                        
                        if (!empty($end_date)) {
                            echo ' - ' . esc_html($end_date);
                        }
                        ?>

                        <?php if (vizou_event_time()): ?>
                            <span class="event-time"><?php vizou_event_time(); ?></span>
                        <?php endif; ?>
                    </p>

                    <!-- TITLE (VENUE, ARTIST) -->
                    <h1>
                        <?php 
                        // Display format depends on the setting
                        $display_format = Vizou_Events_Settings::get_option('display_format');
                        
                        if ($display_format === 'artist_first') {
                            // Artist first
                            vizou_event_artist();
                            echo ' <span class="event-venue">';
                            vizou_event_venue();
                            echo '</span>';
                        } else {
                            // Venue first (default)
                            vizou_event_venue();
                            echo ' <span class="event-artist">';
                            vizou_event_artist();
                            echo '</span>';
                        }
                        ?>
                    </h1>

                    <!-- LOCATION -->
                    <p class="event-details">
                        <span class="event-address">
                                <?php if (vizou_venue_street(null, false)): ?>
                                    <span class="event-street"><?php vizou_venue_street(); ?>, </span>
                                <?php endif; ?>
                                
                                <?php if (vizou_venue_city(null, false)): ?>
                                    <span class="event-city"><?php vizou_venue_city(); ?></span>
                                <?php endif; ?>
                                
                                <?php if (vizou_venue_province(null, false)): ?>
                                    , <span class="event-province"><?php vizou_venue_province(); ?></span>
                                <?php endif; ?>
                                
                                <?php if (vizou_venue_country(null, false)): ?>
                                    <span class="event-country"> (<?php vizou_venue_country(); ?>)</span>
                                <?php endif; ?>
                            </span>
                    </p>

                    <!-- NOTES -->
                    <?php if (vizou_event_notes(null, false)): ?>
                        <div class="event-notes">
                            <?php vizou_event_notes(); ?>
                        </div>
                    <?php endif; ?>

                    <!-- THUMBNAIL -->
                    <?php if ((function_exists('has_post_thumbnail')) && (has_post_thumbnail())) : ?>
                        <a href="<?php echo esc_url(get_permalink()); ?>" title="<?php echo esc_attr(get_the_title()); ?>">
                            <img class="event-thumb" 
                                src="<?php echo esc_url($thumb[0]); ?>" 
                                width="<?php echo esc_attr($thumb[1]); ?>" 
                                height="<?php echo esc_attr($thumb[2]); ?>" 
                                alt="<?php echo esc_attr($thumb_alt); ?>"
                                loading="lazy"/>
                        </a>
                    <?php endif; ?>  

                    <!-- EVENT STATUS -->
                    <?php vizou_event_status(); ?>

                </article>

                <?php edit_post_link(esc_html__('Edit', 'vizou-events-manager'), '', '');?>

            </main>						
        </div>
                    
    <?php endwhile; endif; ?>
        
<?php get_footer(); ?>